package sample.rule;

import com.codeborne.selenide.Selenide;
import com.codeborne.selenide.WebDriverRunner;
import org.junit.rules.TestRule;
import org.junit.runner.Description;
import org.junit.runners.model.Statement;

import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public final class Settings implements TestRule {

    private static final Runnable browserSetting = () -> {
        if (System.getProperty("selenide.browser", System.getProperty("browser")) == null) {
            System.setProperty("selenide.browser", WebDriverRunner.CHROME);
        }
        if (System.getProperty("webdriver.chrome.driver") == null) {
            System.setProperty("webdriver.chrome.driver", "libs/chromedriver");
        }
    };

    private static final Runnable clearBrowserCookies = Selenide::clearBrowserCookies;

    private final List<Runnable> actions;

    private Settings(final List<Runnable> actions) {
        this.actions = Objects.requireNonNull(actions);
    }

    @Override
    public Statement apply(final Statement base, final Description description) {
        return new Statement() {
            @Override
            public void evaluate() throws Throwable {
                actions.forEach(Runnable::run);
                base.evaluate();
            }
        };
    }

    public Settings clearBrowserCookies() {
        return new Settings(Stream.concat(actions.stream(), Stream.of(clearBrowserCookies))
                .collect(Collectors.toList()));
    }

    public static Settings defaultSettings() {
        return new Settings(Collections.singletonList(browserSetting));
    }

    public static Settings remoteSettings() {
        return new Settings(Collections.singletonList(() -> {
            System.setProperty("selenide.browser", WebDriverRunner.FIREFOX);
            System.setProperty("remote", "http://localhost:4444/wd/hub");
        }));
    }
}
