package sample.test;

import static com.codeborne.selenide.Condition.*;

import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.boot.context.embedded.LocalServerPort;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.context.SpringBootTest.WebEnvironment;
import org.springframework.test.context.junit4.SpringRunner;

import sample.page.HelloPage;
import sample.page.LoginPage;
import sample.rule.Retry;
import sample.rule.Settings;

/**
 * Spring Bootアプリケーションで作られたログイン画面をテストします。
 * 
 * テスト開始時に自動でSpring Bootアプリケーションが起動します。
 *
 */
@RunWith(SpringRunner.class)
@SpringBootTest(webEnvironment = WebEnvironment.RANDOM_PORT)
public class LoginTest {

    @LocalServerPort
    public void setPort(int port) {
        System.setProperty("selenide.baseUrl", "http://localhost:" + port);
    }

    @Rule
    public Retry retry = new Retry();

    @Rule
    public Settings settings = Settings.defaultSettings().clearBrowserCookies();

    /**
     * 初期表示時のテキストフィールドなどの状態をテストします。
     * 
     * @throws Exception
     */
    @Test
    public void testInitialized() throws Exception {
        LoginPage loginPage = LoginPage.openPage();
        loginPage
                .errorMessageShouldBe(hidden)
                .usernameShouldBe(empty)
                .passwordShouldBe(empty);
    }

    /**
     * ログインが成功することをテストします。
     * 
     * @throws Exception
     */
    @Test
    public void testLoginSuccessful() throws Exception {
        LoginPage loginPage = LoginPage.openPage();
        HelloPage helloPage = loginPage
                .setUsername("uragami")
                .setPassword("secret")
                .login();
        helloPage.messageShouldBe(exactText("ようこそ、uragamiさん"));
    }

    /**
     * ログインが失敗することをテストします。
     * 
     * @throws Exception
     */
    @Test
    public void testLoginFailure() throws Exception {
        LoginPage loginPage = LoginPage.openPage();
        loginPage
                .setUsername("mistake")
                .setPassword("secret")
                .loginFailure()
                .errorMessageShouldBe(not(empty));
    }
}
